﻿using Autofac;
using SyncFusionDemo1.Attributes;
using SyncFusionDemo1.Helpers;
using SyncFusionDemo1.Interfaces.Services;
using SyncFusionDemo1.Resources;
using SyncFusionDemo1.View.Pages;
using SyncFusionDemo1.ViewModels.Pages;
using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace SyncFusionDemo1.Services
{
    public class NavigationService : INavigationService
    {
        public Task Init()
        {
            return Init(Application.Current.MainPage);
        }

        private static Task Init(Page page)
        {
            page = GetPage(page);

            if (page is TabbedPage tabbedPage)
            {
                foreach (var childPage in tabbedPage.Children)
                {
                    Init(childPage);
                }
            }

            if (!(page.BindingContext is BasePageViewModel viewModel)) return Task.CompletedTask;

            async void AppearingEventHandler(object sender, EventArgs args)
            {
                await viewModel.OnAppearing();
            }

            async void DisappearingEventHandler(object sender, EventArgs args)
            {
                await viewModel.OnDisappearing();
            }

            page.Appearing += AppearingEventHandler;
            page.Disappearing += DisappearingEventHandler;

            return Task.CompletedTask;
        }

        private static Page GetPage(Page page)
        {
            switch (page)
            {
                case MasterDetailPage masterDetailPage:
                    page = GetPage(masterDetailPage.Detail);
                    break;
                case NavigationPage mainNavigationPage:
                    page = mainNavigationPage.CurrentPage;
                    break;
            }

            return page;
        }

        public Task ShowPageAsync<TViewModel>() where TViewModel : BasePageViewModel
        {
            var newPage = CreatePage<TViewModel>();

            Init(newPage);

            var page = Application.Current.MainPage;

            if (page == null) return Task.CompletedTask;

            page = GetPage(page);

            return page.Navigation.PushAsync(newPage);

        }

        public Task ShowPageAsync<TViewModel>(object parameter) where TViewModel : BasePageViewModel
        {
            var newPage = CreatePage<TViewModel>();

            Init(newPage);

            if (newPage.BindingContext is BasePageViewModel viewModel)
            {
                viewModel.Init(parameter);
            }

            var page = Application.Current.MainPage;

            if (page == null) return Task.CompletedTask;

            page = GetPage(page);

            return page.Navigation.PushAsync(newPage);
        }

        public Task ShowPageAsync(Type type)
        {
            var newPage = CreatePage(type);

            Application.Current.MainPage = new MasterDetailPage
            {
                Detail = new NavigationPage(newPage),
                Master = new MasterDetailPageMaster()
            };

            return Init();
        }

        public Task CloseAllPagesAsync(bool animated)
        {
            var page = Application.Current.MainPage;

            if (page == null) return Task.CompletedTask;

            page = GetPage(page);

            return page.Navigation.PopToRootAsync(animated);
        }

        //public Task ShowPopupAsync<TViewModel>() where TViewModel : BasePageViewModel
        //{
        //    var newPage = CreatePage<TViewModel>();

        //    Init(newPage);

        //    var page = Application.Current.MainPage;

        //    if (page == null) return Task.CompletedTask;

        //    page = GetPage(page);

        //    return page.Navigation.PushPopupAsync(newPage as PopupPage);
        //}

        public Task AppendToTitle(string value)
        {
            var page = GetPage(Application.Current.MainPage);

            page.Title += value;

            return Task.CompletedTask;
        }

        public Task ClosePageAsync(bool animated)
        {
            var page = Application.Current.MainPage;

            if (page == null) return Task.CompletedTask;

            switch (page)
            {
                case NavigationPage mainNavigationPage:
                    page = mainNavigationPage.CurrentPage;
                    break;
            }

            return page.Navigation.PopAsync(animated);
        }

        //public Task ClosePopupAsync(bool animated = false)
        //{
        //    var page = Application.Current.MainPage;

        //    if (page == null) return Task.CompletedTask;

        //    switch (page)
        //    {
        //        case NavigationPage mainNavigationPage:
        //            page = mainNavigationPage.CurrentPage;
        //            break;
        //    }

        //    return page.Navigation.PopPopupAsync(animated);
        //}

        private Page CreatePage<TViewModel>() where TViewModel : BasePageViewModel
        {
            var attribute = typeof(TViewModel).GetCustomAttribute<PageAttribute>(false);

            //TODO fix the TemplateRenderer
            if (attribute == null)
            {
                throw new Exception(TemplateRenderer.Render(NavigationResources.PageMissing, new { PageAttribute = nameof(PageAttribute) }));
            }

            if (attribute.Type == null)
            {
                throw new Exception(TemplateRenderer.Render(NavigationResources.PageTypeMissing, new { PageAttribute = nameof(PageAttribute), PageType = nameof(PageAttribute.Type) }));
            }

            var page = Activator.CreateInstance(attribute.Type) as Page;

            if (page != null && page.BindingContext == null)
            {
                page.BindingContext = DI.Container.Resolve<TViewModel>();
            }

            return page;
        }

        private Page CreatePage(Type type)
        {
            var attribute = type.GetCustomAttribute<PageAttribute>(false);

            //TODO fix the TemplateRenderer
            if (attribute == null)
            {
                throw new Exception(TemplateRenderer.Render(NavigationResources.PageMissing, new { PageAttribute = nameof(PageAttribute) }));
            }

            if (attribute.Type == null)
            {
                throw new Exception(TemplateRenderer.Render(NavigationResources.PageTypeMissing, new { PageAttribute = nameof(PageAttribute), PageType = nameof(PageAttribute.Type) }));
            }

            var page = Activator.CreateInstance(attribute.Type) as Page;

            if (page != null && page.BindingContext == null)
            {
                page.BindingContext = DI.Container.Resolve(type);
            }

            return page;
        }
    }
}
